;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Stationary Magikoopa, by yoshicookiezeus
; 
; Description: This Magikoopa sits in one place repeatedly firing magic,
; disappearing whenever Mario gets near and reappearing when he goes away.
; Unlike the normal Magikoopa, several of these can be active at once,
; and after getting killed (by a thrown sprite, for instance) it does
; not respawn.
; Note that wherever you place the sprite in Lunar Magic is where its head
; appears, so it should be placed one tile above the floor you want it to
; stand on.
;
; Uses the extra bit: No
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite constants - feel free to change these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!SpriteToGen		= $20			; Magikoopa magic
	!WandTile		= $99

Tilemap:
	db $A0,$C0,$A0,$C0,$A4,$C4,$A4,$C4,$A0,$C0,$A0,$C0
OffsetY:
	db $10,$00

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ROM and RAM defines - don't mess with these
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	!RAM_FrameCounter	= $13
	!RAM_FrameCounterB	= $14
	!RAM_ScreenBndryXLo	= $1A
	!RAM_ScreenBndryXHi	= $1B
	!RAM_ScreenBndryYLo	= $1C
	!RAM_ScreenBndryYHi	= $1D
	!RAM_MarioSpeedX	= $7B
	!RAM_MarioXPos		= $94
	!RAM_MarioXPosHi	= $95
	!RAM_SpritesLocked	= $9D
	!RAM_SpriteNum		= $9E
	!RAM_SpriteSpeedY	= $AA
	!RAM_SpriteSpeedX	= $B6
	!RAM_SpriteState	= $C2
	!RAM_SpriteYLo		= $D8
	!RAM_SpriteXLo		= $E4
	!OAM_DispX		= $0300
	!OAM_DispY		= $0301
	!OAM_Tile		= $0302
	!OAM_Prop		= $0303
	!OAM_Tile2DispX		= $0304
	!OAM_Tile2DispY		= $0305
	!OAM_Tile2		= $0306
	!OAM_Tile2Prop		= $0307
	!OAM_Tile3DispX		= $0308
	!OAM_Tile3DispY		= $0309
	!OAM_Tile3		= $030A
	!OAM_Tile3Prop		= $030B
	!OAM_TileSize		= $0460
	!RAM_SpriteStatus	= $14C8
	!RAM_SpriteYHi		= $14D4
	!RAM_SpriteXHi		= $14E0
	!RAM_SpriteDir		= $157C
	!RAM_SprObjStatus	= $1588
	!RAM_OffscreenHorz	= $15A0
	!RAM_CurrentSprIndex	= $15E9
	!RAM_SprOAMIndex	= $15EA
	!RAM_SpritePal		= $15F6
	!RAM_SmokeNum		= $17C0
	!RAM_SmokeYLo		= $17C4
	!RAM_SmokeXLo		= $17C8
	!RAM_SmokeTimer		= $17CC
	!RAM_OffscreenVert	= $186C

	!FinishOAMWrite		= $01B7B3
	!InitSpriteTables	= $07F7D2

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite init JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc
	RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; sprite code JSL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "MAIN ",pc
	PHB
	PHK
	PLB
	JSR Main
	PLB
	RTL


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:
	JSR SubOffScreenX0
	LDA #$01
	STA $15D0,x
	LDA !RAM_SpriteState,x
	JSL $0086DF

MagiKoopaPtrs:
	dw Gone
	dw Attacking

Gone:
	LDA !RAM_SpritesLocked		;\ if sprites locked,
	BNE .Return			;/ return
	JSR CheckProximityMario
	BNE .Return

	INC !RAM_SpriteState,x		; go to next sprite state
	STZ $1570,x
	JSR SubHorzPos			;\ make sprite face Mario
	TYA				; |
	STA !RAM_SpriteDir,x		;/
	JSR SpawnSmoke
	LDA #$70
	STA $1540,x
.Return
	RTS				; return


DATA_01BE69:
	db $04,$02,$00

DATA_01BE6C:
	db $10,$F8

Attacking:
	STZ $15D0,x
	JSL $01803A			; interact with Mario and with other sprites
	JSR SubHorzPos			;\ make sprite face Mario
	TYA				; |
	STA !RAM_SpriteDir,x		;/
	LDA $1540,x			;\ if attack cycle not complete,
	BNE .DontReset			;/ branch
	LDA #$70			;\ reset cycle
	STA $1540,x			;/
.DontReset
	JSR CheckProximityMario
	BEQ .DontDisappear
	STZ !RAM_SpriteState,x		; go to next sprite state
	JSR SpawnSmoke
	BRA .SetGraphics

.DontDisappear
	LDA $1540,x
	CMP #$40			;\ if not time to generate magic,
	BNE .SetGraphics		;/ branch
	PHA				; preserve sprite state timer
	LDA !RAM_SpritesLocked		;\ if sprites locked
	ORA !RAM_OffscreenHorz,x	; | or sprite is offscreen,
	BNE .DontSpawn			;/ branch
	JSR SpawnMagic
.DontSpawn
	PLA				; retrieve sprite state timer

.SetGraphics
	LSR				;\ use sprite state timer to determine graphics frame to use
	LSR				; | in some very complicated manner
	LSR				; |
	LSR				; |
	LSR				; |
	LSR				; |
	TAY				; | get two highest bits of sprite state timer into y register
	PHY				; | and preserve them
	LDA $1540,x			; |
	LSR				; |
	LSR				; |
	LSR				; |
	AND #$01			; | get fourth bit of sprite state timer
	ORA DATA_01BE69,y		; | add in something determined by two highest bits
	STA $1602,x			;/ and use it to determine sprite graphics frame to use

	JSR SpriteGraphics

	LDA $1602,x			;\ if sprite graphics frame less than 4,
	SEC				; |
	SBC #$02			; |
	CMP #$02			; |
	BCC .Label1			;/ branch
	LSR				;\ if it's less than 
	BCC .Label1			;/ branch
	LDA !RAM_SprOAMIndex,x		;\ place head tile one pixel lower
	TAX				; |
	INC !OAM_DispY,x		;/

	LDX !RAM_CurrentSprIndex	; load sprite index

.Label1
	PLY				;\ retrieve seventh bit of sprite state timer
	CPY #$01			; | if it's clear,
	BNE DisplayWand			;/ branch
	JSR CODE_01B14E			; sparkle effect

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

DisplayWand:
	LDA $1602,x			;\ if sprite graphics frame less than 4,
	CMP #$04			; |
	BCC .Return			;/ return
	LDY !RAM_SpriteDir,x		;\ use sprite direction to determine x position of wand tile
	LDA !RAM_SpriteXLo,x		; |
	CLC				; |
	ADC DATA_01BE6C,y		; |
	SEC				; |
	SBC !RAM_ScreenBndryXLo		; |
	LDY !RAM_SprOAMIndex,x		; |
	STA !OAM_Tile3DispX,y		;/
	LDA !RAM_SpriteYLo,x		;\ set y position of wand tile
	SEC				; |
	SBC !RAM_ScreenBndryYLo		; |
	CLC				; |
	ADC #$10			; |
	STA !OAM_Tile3DispY,y		;/
	LDA !RAM_SpriteDir,x		;\ set properties of wand tile
	LSR				; |
	LDA #$00			; |
	BCS .NoFlip			; |
	ORA #$40			; |
.NoFlip					; |
	ORA $64				; |
	ORA !RAM_SpritePal,x		; |
	STA !OAM_Tile3Prop,y		;/
	LDA #!WandTile			;\ set wand tile number
	STA !OAM_Tile3,y		;/
	TYA				;\ set wand tile size
	LSR				; |
	LSR				; |
	TAY				; |
	LDA #$00			; |
	ORA !RAM_OffscreenHorz,x	; |
	STA $0462,y			;/
.Return
	RTS				; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CheckProximityMario:
	JSR SubHorzPos			;\ if sprite not closer to Mario than 0x20 pixels horizontally,
	LDA $0F				; |
	CLC				; |
	ADC #$30			; |
	CMP #$60			; |
	BCS .FarAway			;/ branch
	JSR SubVertPos			;\ if sprite not closer to Mario than 0x20 pixels horizontally,
	LDA $0F				; |
	CLC				; |
	ADC #$30			; |
	CMP #$60			; |
	BCS .FarAway			;/ branch
	LDA #$01
	RTS
.FarAway
	LDA #$00
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpawnMagic:
	LDY #$09			; setup loop
.LoopStart
	LDA $14C8,y			;\ check if sprite slot is free
	BEQ .FoundSlot			;/ if so, branch
	DEY				;\ if not, check next slot
	BPL .LoopStart			;/
	RTS				; if no slots left, return

.FoundSlot
	LDA #$10			;\ sound effect
	STA $1DF9			;/ 
	LDA #$08			;\ set sprite status
	STA !RAM_SpriteStatus,y		;/ 
	LDA #!SpriteToGen		;\ set sprite number       
	STA.w !RAM_SpriteNum,y		;/
	LDA !RAM_SpriteXLo,x		;\ set sprite x position
	STA.w !RAM_SpriteXLo,y		; |
	LDA !RAM_SpriteXHi,x		; |
	STA !RAM_SpriteXHi,y		;/
	LDA !RAM_SpriteYLo,x		;\ set sprite y position
	CLC				; |
	ADC #$0A			; |
	STA.w !RAM_SpriteYLo,y		; |
	LDA !RAM_SpriteYHi,x		; |
	ADC #$00			; |
	STA !RAM_SpriteYHi,y		;/
	TYX                       
	JSL !InitSpriteTables		; clear out old sprite values
	LDA #$20
	JSR CODE_01BF6A			; aiming routine
	LDX !RAM_CurrentSprIndex
	LDA $00				;\ set sprite speeds
	STA.w !RAM_SpriteSpeedY,y	; |
	LDA $01				; |
	STA.w !RAM_SpriteSpeedX,y	;/
	RTS				; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Graphics routine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


SpriteGraphics:
	JSR GetDrawInfo
	LDA $1602,x			;\ use graphics frame to determine initial tile table offset
	ASL				; |
	STA $03				;/
	LDA !RAM_SpriteDir,x
	STA $02
	PHX				; preserve sprite index
	LDX #$01			; setup loop counter
.LoopStart
	LDA $01				;\ set y position of tile
	CLC				; |
	ADC OffsetY,x			; |
	STA !OAM_DispY,y		;/
	LDA $00				;\ set x position of tile
	STA !OAM_DispX,y		;/
	PHX				; preserve loop counter
	LDX $03				; get tilemap index
	LDA Tilemap,x			;\ set tile number
	STA !OAM_Tile,y			;/
	LDX !RAM_CurrentSprIndex	; load sprite index
	LDA !RAM_SpritePal,x		; load sprite graphics properties
	PLX				; retrieve loop counter
	PHY				; preserve OAM index
	LDY $02				;\ if sprite facing right,
	BNE .NoFlip			; |
	EOR #$40			;/ flip tile
.NoFlip
	PLY				; retrieve OAM index
	ORA $64				; add in level properties
	STA !OAM_Prop,Y			; set tile properties
	INY				;\ increase OAM index so that the next tile can be written
	INY				; |
	INY				; |
	INY				;/
	INC $03
	DEX				; decrease loop counter
	BPL .LoopStart			; if still tiles left to draw, go to start of loop
	PLX				; retrieve sprite index
	LDY #$02			; the tiles written were 16x16
	LDA #$01			; we wrote two tiles
	JSL !FinishOAMWrite
	RTS				; return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; aiming routine
; input: accumulator should be set to total speed (x+y)
; output: $00 = y speed, $01 = x speed
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

CODE_01BF6A:
	STA $01
	PHX			;\ preserve sprite indexes of Magikoopa and magic
	PHY			;/
	JSR CODE_01AD42		; $0E = vertical distance to Mario
	STY $02			; $02 = vertical direction to Mario
	LDA $0E			;\ $0C = vertical distance to Mario, positive
	BPL CODE_01BF7C		; |
	EOR #$FF		; |
	CLC			; |
	ADC #$01		; |
CODE_01BF7C:			; |
	STA $0C			;/
	JSR SubHorzPos		; $0F = horizontal distance to Mario
	STY $03			; $03 = horizontal direction to Mario
	LDA $0F			;\ $0D = horizontal distance to Mario, positive
	BPL CODE_01BF8C		; |
	EOR #$FF		; |
	CLC			; |
	ADC #$01		; |
CODE_01BF8C:			; |
	STA $0D			;/
	LDY #$00
	LDA $0D			;\ if vertical distance less than horizontal distance,
	CMP $0C			; |
	BCS CODE_01BF9F		;/ branch
	INY			; set y register
	PHA			;\ switch $0C and $0D
	LDA $0C			; |
	STA $0D			; |
	PLA			; |
	STA $0C			;/
CODE_01BF9F:
	LDA #$00		;\ zero out $00 and $0B
	STA $0B			; | ...what's wrong with STZ?
	STA $00			;/
	LDX $01			;\ divide $0C by $0D?
CODE_01BFA7:			; |
	LDA $0B			; |\ if $0C + loop counter is less than $0D,
	CLC			; | |
	ADC $0C			; | |
	CMP $0D			; | |
	BCC CODE_01BFB4		; |/ branch
	SBC $0D			; | else, subtract $0D
	INC $00			; | and increase $00
CODE_01BFB4:			; |
	STA $0B			; |
	DEX			; |\ if still cycles left to run,
	BNE CODE_01BFA7		;/ / go to start of loop
	TYA			;\ if $0C and $0D was not switched,
	BEQ CODE_01BFC6		;/ branch
	LDA $00			;\ else, switch $00 and $01
	PHA			; |
	LDA $01			; |
	STA $00			; |
	PLA			; |
	STA $01			;/
CODE_01BFC6:
	LDA $00			;\ if horizontal distance was inverted,
	LDY $02			; | invert $00
	BEQ CODE_01BFD3		; |
	EOR #$FF		; |
	CLC			; |
	ADC #$01		; |
	STA $00			;/
CODE_01BFD3:
	LDA $01			;\ if vertical distance was inverted,
	LDY $03			; | invert $01
	BEQ CODE_01BFE0		; |
	EOR #$FF		; |
	CLC			; |
	ADC #$01		; |
	STA $01			;/
CODE_01BFE0:
	PLY			;\ retrieve Magikoopa and magic sprite indexes
	PLX			;/
	RTS			; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; display smoke effect
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SpawnSmoke:
	
	LDA !RAM_OffscreenVert,x
	ORA !RAM_OffscreenHorz,x
	BNE .Return

	LDY #$03			; find a free slot to display effect
.LoopStart
	LDA !RAM_SmokeNum,y
	BEQ .FoundSlot
	DEY
	BPL .LoopStart
.Return
	RTS				; return if no slots open

.FoundSlot
	LDA #$01			;\ set effect graphic to smoke graphic
	STA !RAM_SmokeNum,y		;/
	LDA !RAM_SpriteYLo,x		;\ set smoke y position
	CLC				; |
	ADC #$08			; |
	STA !RAM_SmokeYLo,y		;/
	LDA !RAM_SpriteXLo,x		;\ smoke x position = shooter x position
	STA !RAM_SmokeXLo,y		;/
	LDA #$1B			;\ set time to show smoke
	STA !RAM_SmokeTimer,y		;/
	RTS	 


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


CODE_01B14E:
	LDA !RAM_FrameCounter		;\ only run code every fourth frame
	AND #$03			; |
	ORA !RAM_OffscreenVert,x	; | if sprite offscreen vertically
	ORA !RAM_SpritesLocked		; | or sprites locked,
	BNE Return01B191		;/ return
	JSL $01ACF9			;\ #$02 = sprite xpos low byte + random number between 0x-5 and 0xB
	AND #$0F			; |
	CLC				; |
	LDY #$00			; |
	ADC #$FC			; |
	BPL CODE_01B167			; |
	DEY				; |
CODE_01B167:				; |
	CLC				; |
	ADC !RAM_SpriteXLo,x		; |
	STA $02				;/
	TYA				;\ if $02 means an offscreen location?
	ADC !RAM_SpriteXHi,x		; |
	PHA				; |
	LDA $02				; |
	CMP !RAM_ScreenBndryXLo		; |
	PLA				; |
	SBC !RAM_ScreenBndryXHi		; |
	BNE Return01B191		;/ return
	LDA $148E			;\ #$00 = sprite ypos low byte + random number between 0x-2 and 0xD
	AND #$0F			; |
	CLC				; |
	ADC #$FE			; |
	ADC !RAM_SpriteYLo,x		; |
	STA $00				;/
	LDA !RAM_SpriteYHi,x		;\ #$01 = sprite ypos high byte with changes for earlier random number
	ADC #$00			; |
	STA $01				;/
	JSL $0285BA			; sparkle effect
Return01B191:
	RTS				; return



CODE_01AD42:
	LDY #$00
	LDA $D3
	CLC			;\ make subroutine use position of Mario's lower half instead of the upper one
	ADC #$08		;/ this wasn't in the original routine
	SEC
	SBC !RAM_SpriteYLo,x
	STA $0E
	LDA $D4
	SBC !RAM_SpriteYHi,x
	BPL Return01AD53
	INY
Return01AD53:
	RTS			; return


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GetDrawInfo
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B75C

Table1:	db $0C,$1C
Table2:	db $01,$02

GetDrawInfo:
	STZ $186C,x		; reset sprite offscreen flag, vertical
	STZ $15A0,x		; reset sprite offscreen flag, horizontal
	LDA $E4,x		;\
	CMP $1A			; | set horizontal offscreen if necessary
	LDA $14E0,x		; |
	SBC $1B			; |
	BEQ .OnScreenX		; |
	INC $15A0,x		;/

.OnScreenX
	LDA $14E0,x		;\
	XBA			; |
	LDA $E4,x		; |
	REP #$20		; |
	SEC			; |
	SBC $1A			; | mark sprite invalid if far enough off screen
	CLC			; |
	ADC #$0040		; |
	CMP #$0180		; |
	SEP #$20		; |
	ROL A			; |
	AND #$01		; |
	STA $15C4,x		;/ 
	BNE .Invalid		; 

	LDY #$00		;\ set up loop:
	LDA $1662,x		; | 
	AND #$20		; | if not smushed (1662 & 0x20), go through loop twice
	BEQ .OnScreenLoop	; | else, go through loop once
	INY			;/ 
.OnScreenLoop
	LDA $D8,x		;\ 
	CLC			; | set vertical offscreen if necessary
	ADC Table1,y		; |
	PHP			; |
	CMP $1C			; | (vert screen boundary)
	ROL $00			; |
	PLP			; |
	LDA $14D4,x		; | 
	ADC #$00		; |
	LSR $00			; |
	SBC $1D			; |
	BEQ .OnScreenY		; |
	LDA $186C,x		; | (vert offscreen)
	ORA Table2,y		; |
	STA $186C,x		; |
.OnScreenY
	DEY			; |
	BPL .OnScreenLoop	;/

	LDY $15EA,x		; get offset to sprite OAM
	LDA $E4,x		;\ 
	SEC			; | 
	SBC $1A			; | $00 = sprite x position relative to screen boarder
	STA $00			;/ 
	LDA $D8,x		;\ 
	SEC			; | 
	SBC $1C			; | $01 = sprite y position relative to screen boarder
	STA $01			;/ 
	RTS			; return

.Invalid
	PLA			;\ return from *main gfx routine* subroutine...
	PLA			; |    ...(not just this subroutine)
	RTS			;/

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubOffScreen
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                    
Spr_T12:
	db $40,$B0
Spr_T13:
	db $01,$FF
Spr_T14:
	db $30,$C0,$A0,$C0,$A0,$F0,$60,$90		;bank 1 sizes
	db $30,$C0,$A0,$80,$A0,$40,$60,$B0		;bank 3 sizes
Spr_T15:
	db $01,$FF,$01,$FF,$01,$FF,$01,$FF		;bank 1 sizes
	db $01,$FF,$01,$FF,$01,$00,$01,$FF		;bank 3 sizes

SubOffScreenX1:
	LDA #$02		;\ entry point of routine determines value of $03
	BRA Store03		; | (table entry to use on horizontal levels)
SubOffScreenX2:
	LDA #$04		; | 
	BRA Store03		; |
SubOffScreenX3:
	LDA #$06		; |
	BRA Store03		; |
SubOffScreenX4:
	LDA #$08		; |
	BRA Store03		; |
SubOffScreenX5:
	LDA #$0A		; |
	BRA Store03		; |
SubOffScreenX6:
	LDA #$0C		; |
	BRA Store03		; |
SubOffScreenX7:
	LDA #$0E		; |
Store03:
	STA $03			; |            
	BRA StartSub		; |
SubOffScreenX0:
	STZ $03			;/

StartSub:
	JSR SubIsOffScreen	;\ if sprite is not off screen, return
	BEQ .Return		;/
	LDA $5B			;\  goto VERTICAL_LEVEL if vertical level
	AND #$01		; |
	BNE .VerticalLevel	;/     
	LDA $D8,x		;\
	CLC			; | 
	ADC #$50		; | if the sprite has gone off the bottom of the level...
	LDA $14D4,x		; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
	ADC #$00		; | 
	CMP #$02		; | 
	BPL .EraseSprite	;/    ...erase the sprite
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13
	AND #$01
	ORA $03
	STA $01
	TAY
	LDA $1A
	CLC
	ADC Spr_T14,y
	ROL $00
	CMP $E4,x
	PHP
	LDA $1B
	LSR $00
	ADC Spr_T15,y
	PLP
	SBC $14E0,x
	STA $00
	LSR $01
	BCC .Label31
	EOR #$80
	STA $00
.Label31
	LDA $00
	BPL .Return
.EraseSprite
	LDA $14C8,x		;\ if sprite status < 8, permanently erase sprite
	CMP #$08		; |
	BCC .KillSprite		;/    
	LDY $161A,x
	CPY #$FF
	BEQ .KillSprite
	LDA #$00
	STA $1938,y
.KillSprite
	STZ $14C8,x		; erase sprite
.Return
	RTS			; return

.VerticalLevel
	LDA $167A,x		;\ if "process offscreen" flag is set, return
	AND #$04		; |
	BNE .Return		;/
	LDA $13			;\
	LSR A			; | 
	BCS .Return		;/
	LDA $E4,x		;\ 
	CMP #$00		; | if the sprite has gone off the side of the level...
	LDA $14E0,x		; |
	SBC #$00		; |
	CMP #$02		; |
	BCS .EraseSprite	;/  ...erase the sprite
	LDA $13
	LSR A
	AND #$01
	STA $01
	TAY
	LDA $1C
	CLC
	ADC Spr_T12,y
	ROL $00
	CMP $D8,x
	PHP
	LDA $001D
	LSR $00
	ADC Spr_T13,y
	PLP
	SBC $14D4,x
	STA $00
	LDY $01
	BEQ .Label38
	EOR #$80
	STA $00
.Label38
	LDA $00
	BPL .Return
	BMI .EraseSprite

SubIsOffScreen:
	LDA $15A0,x		;\ if sprite is on screen, accumulator = 0 
	ORA $186C,x		; |  
	RTS			;/ return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubHorzPos
; This routine determines which side of the sprite Mario is on.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B817
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;org $03B817

SubHorzPos:
	LDY #$00
	LDA $94
	SEC
	SBC $E4,x
	STA $0F
	LDA $95
	SBC $14E0,x
	BPL .Return
	INY
.Return
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SubVertPos
; This routine determines if Mario is above or below the sprite.  It sets the Y register
; to the direction such that the sprite would face Mario
; It is ripped from $03B829
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	;org $03B817

SubVertPos:
	LDY #$00
	LDA $D3
	SEC
	SBC $D8,x
	STA $0F
	LDA $D4
	SBC $14D4,x
	BPL .Return
	INY
.Return
	RTS